/*
 * LinkedList.cpp implements the LinkedList class behavior declared in LinkedList.h.
 */

#include <string>
#include <stdexcept>
#include "LinkedList.h"


LinkedList::LinkedList() {}

LinkedList::~LinkedList() {
    clear();
}

void LinkedList::add(int value) {
    if (m_front == nullptr) {
        // empty list: add this node as the new front
        m_front = new ListNode(value, nullptr);
    } else {
        // non-empty list: move to end, attach new node
        ListNode* current = m_front;
        while (current->next != nullptr) {
            current = current->next;
        }
        current->next = new ListNode(value, nullptr);
    }
}

void LinkedList::clear() {
    while (m_front != nullptr) {
        ListNode* temp = m_front;
        m_front = m_front->next;
        delete temp;
    }
}

int LinkedList::get(int index) const {
    checkIndex(index, 0, size() - 1);
    ListNode* current = m_front;
    for (int i = 0; i < index; i++) {
        current = current->next;
    }
    return current->data;
}

void LinkedList::insert(int index, int value) {
    checkIndex(index, 0, size());
    if (index == 0) {
        ListNode* temp = m_front;
        m_front = new ListNode(value, temp);
    } else {
        ListNode* current = m_front;
        for (int i = 0; i < index - 1; i++) {
            current = current->next;
        }
        ListNode* temp = current->next;
        current->next = new ListNode(value, temp);
    }
}

bool LinkedList::isEmpty() const {
    return m_front == nullptr;
}

void LinkedList::remove(int index) {
    checkIndex(index, 0, size() - 1);
    ListNode* trash;
    if (index == 0) {
        trash = m_front;
        m_front = m_front->next;
    } else {
        ListNode* curr = m_front;
        for (int i = 0; i < index-1; i++) {
            curr = curr->next;
        }
        trash = curr->next;
        curr->next = curr->next->next;
    }
    delete trash;
}

void LinkedList::set(int index, int value) {
    checkIndex(index, 0, size() - 1);
    ListNode* current = m_front;
    for (int i = 0; i < index; i++) {
        current = current->next;
    }
    current->data = value;
}

int LinkedList::size() const {
    int count = 0;
    ListNode* current = m_front;
    while (current != nullptr) {
        count++;
        current = current->next;
    }
    return count;
}

string LinkedList::toString() const {
    string result = "[";
    if (!isEmpty()) {
        result += to_string(m_front->data);
        ListNode* curr = m_front->next;
        while (curr != nullptr) {
            result += ", " + to_string(curr->data);
            curr = curr->next;
        }
    }
    return result + "]";
}

void LinkedList::checkIndex(int index, int min, int max) const {
    if (index < min || index > max) {
        throw out_of_range("Invalid index");
    }
}
